@@ -1,8 +1,9 @@
 from __future__ import print_function
 from functools import partial
 import itertools
-import time
 import json
+import os
+import time
 
 # Python3 support
 try:
@@ -57,7 +58,40 @@ class BatchError(APIError):
 class GroClient(object):
     """API client with stateful authentication for lib functions and extra convenience methods."""
 
-    def __init__(self, api_host, access_token):
+    def __init__(self, api_host=cfg.API_HOST, access_token=None):
+        """Construct a GroClient instance.
+
+        Parameters
+        ----------
+        api_host : string, optional
+            The API server hostname.
+        access_token : string, optional
+            Your Gro API authentication token. If not specified, the
+            :code:`$GROAPI_TOKEN` environment variable is used. See
+            :doc:`authentication`.
+
+        Raises
+        ------
+            RuntimeError
+                Raised when neither the :code:`access_token` parameter nor
+                :code:`$GROAPI_TOKEN` environment variable are set.
+
+        Examples
+        --------
+            >>> client = GroClient()  # token stored in $GROAPI_TOKEN
+
+            >>> client = GroClient(access_token="your_token_here")
+        """
+        # Initialize early since they're referenced in the destructor and
+        # access_token checking may cause constructor to exit early.
+        self._async_http_client = None
+        self._ioloop = None
+
+        if access_token is None:
+            access_token = os.environ.get("GROAPI_TOKEN")
+            if access_token is None:
+                raise RuntimeError("$GROAPI_TOKEN environment variable must be set when "
+                                   "GroClient is constructed without the access_token argument")
         self.api_host = api_host
         self.access_token = access_token
         self._logger = lib.get_default_logger()
@@ -74,13 +108,13 @@ class GroClient(object):
             self._logger.warning(
                 "Unable to initialize event loop, async methods disabled: {}".format(e)
             )
-            self._async_http_client = None
-            self._ioloop = None
 
     def __del__(self):
-        self._async_http_client.close()
-        self._ioloop.stop()
-        self._ioloop.close()
+        if self._async_http_client is not None:
+            self._async_http_client.close()
+        if self._ioloop is not None:
+            self._ioloop.stop()
+            self._ioloop.close()
 
     def get_logger(self):
         return self._logger
@@ -286,6 +320,15 @@ class GroClient(object):
             points = lib.list_of_series_to_single_series(
                 list_of_series_points, False, include_historical, include_available_date
             )
+            # Apply unit conversion if a unit is specified
+            if "unit_id" in selection:
+                raise gen.Return(list(
+                    map(
+                        partial(self.convert_unit, target_unit_id=selection["unit_id"]),
+                        points,
+                    )
+                ))
+
             raise gen.Return(points)
         except BatchError as b:
             raise gen.Return(b)
@@ -702,6 +745,56 @@ class GroClient(object):
         """
         return lib.get_geojson(self.access_token, self.api_host, region_id, zoom_level)
 
+    def get_ancestor(
+        self,
+        entity_type,
+        entity_id,
+        distance=None,
+        include_details=True,
+    ):
+        """Given an item, metric or region, returns all its ancestors i.e.
+        entities that "contain" in the given entity
+
+        Parameters
+        ----------
+        entity_type : { 'metrics', 'items', 'regions' }
+        entity_id : integer
+        distance: integer, optional
+            Return all entities that contain the entity_id at maximum distance.
+            If not provided, get all ancestors.
+        include_details : boolean, optional
+            True by default. Will perform a lookup() on each ancestor to find name,
+            definition, etc. If this option is set to False, only ids of ancestor
+            entities will be returned, which makes execution significantly faster.
+
+        Returns
+        -------
+        list of dicts
+
+            Example::
+
+                [{
+                    'id': 134,
+                    'name': 'Cattle hides, wet-salted',
+                    'definition': 'Hides and skins of domesticated cattle-animals ...',
+                } , {
+                    'id': 382,
+                    'name': 'Calf skins, wet-salted',
+                    'definition': 'Wet-salted hides and skins of calves-animals of ...'
+                }, ...]
+
+            See output of :meth:`~.lookup`
+
+        """
+        return lib.get_ancestor(
+            self.access_token,
+            self.api_host,
+            entity_type,
+            entity_id,
+            distance,
+            include_details,
+        )
+
     def get_descendant(
         self,
         entity_type,
@@ -712,7 +805,7 @@ class GroClient(object):
         """Given an item, metric or region, returns all its descendants i.e.
         entities that are "contained" in the given entity
 
-        Similar to :meth:~.get_descendant_regions, but also works on items and metrics. This method has
+        Similar to :meth:`~.get_descendant_regions`, but also works on items and metrics. This method has
         a distance parameter (which returns all nested child entities) instead of a descendant_level
         parameter (which only returns child entities at a given depth/level).
 
@@ -756,7 +849,6 @@ class GroClient(object):
             include_details,
         )
 
-
     def get_descendant_regions(
         self,
         region_id,
@@ -903,7 +995,14 @@ class GroClient(object):
             self.access_token, self.api_host, entity_type, num_results, **selection
         )
 
-    def get_df(self, show_revisions=False, show_available_date=False, index_by_series=False, include_names=False):
+    def get_df(
+        self,
+        show_revisions=False,
+        show_available_date=False,
+        index_by_series=False,
+        include_names=False,
+        compress_format=False,
+        async_mode = False):
         """Call :meth:`~.get_data_points` for each saved data series and return as a combined
         dataframe.
 
@@ -913,12 +1012,24 @@ class GroClient(object):
 
         Parameters
         ----------
+            show_revisions : boolean, optional
+                False by default, meaning only the latest value for each period. If true, will return
+                all values for a given period, differentiated by the `reporting_date` field.
+            show_available_date : boolean, optional
+                False by default. If true, will return the available date of each data point.
             index_by_series : boolean, optional
                If set, the dataframe is indexed by series. See https://developers.gro-intelligence.com/data-series-definition.html
             include_names : boolean, optional
                If set, the dataframe will have additional columns with names of entities.
                Note that this will increase the size of the dataframe by about 5x.
-
+            compress_format: boolean, optional
+               If set, each series will be compressed to a single column in the dataframe, with the end_date column
+               set as the dataframe inde. All the entity names for each series will be
+               placed in column headers.
+               compress_format cannot be used simultaneously with show_revisions or show_available_date
+            async_mode: boolean, optional
+                If set, it will make :meth:`~get_data_points` requests asynchronously.
+                Note that when running in a Jupyter Ipython notebook with async_mode, you will need to use nest_asyncio module
         Returns
         -------
         pandas.DataFrame
@@ -926,25 +1037,51 @@ class GroClient(object):
             into a single dataframe.
             See https://developers.gro-intelligence.com/data-point-definition.html
         """
+
+        assert not (
+            compress_format and (show_revisions or show_available_date)
+        ), "compress_format cannot be used simultaneously with show_revisions or show_available_date"
+
+        data_series_list = []
         while self._data_series_queue:
             data_series = self._data_series_queue.pop()
             if show_revisions:
                 data_series["show_revisions"] = True
             if show_available_date:
                 data_series["show_available_date"] = True
-            self.add_points_to_df(
-                None, data_series, self.get_data_points(**data_series)
+            if async_mode:
+                data_series_list.append(data_series)
+            else:
+                self.add_points_to_df(
+                    None, data_series, self.get_data_points(**data_series)
+                )
+
+        if async_mode:
+            self.batch_async_get_data_points(
+                data_series_list,
+                output_list=self._data_frame,
+                map_result=self.add_points_to_df,
             )
 
-        if include_names and not self._data_frame.empty:
-            def get_name(entity_type_id, entity_id):
-                return self.lookup(ENTITY_KEY_TO_TYPE[entity_type_id], entity_id)['name']
+        if compress_format:
+            include_names = True
 
+        if self._data_frame.empty:
+            return self._data_frame
+
+        if include_names:
+            name_cols = []
             for entity_type_id in DATA_SERIES_UNIQUE_TYPES_ID + ['unit_id']:
                 name_col = entity_type_id.replace('_id', '_name')
-                self._data_frame[name_col] = self._data_frame[entity_type_id].apply(partial(get_name, entity_type_id))
+                name_cols.append(name_col)
+                entity_dict = self.lookup(ENTITY_KEY_TO_TYPE[entity_type_id], self._data_frame[entity_type_id].unique())
+                self._data_frame[name_col] = self._data_frame[entity_type_id].apply(lambda entity_id: entity_dict.get(str(entity_id))['name'])
 
-        if index_by_series and not self._data_frame.empty:
+            if compress_format:
+                return self._data_frame.pivot_table(index='end_date', values='value',
+                                                    columns=name_cols)
+
+        if index_by_series:
             idx_columns = intersect(DATA_SERIES_UNIQUE_TYPES_ID, self._data_frame.columns)
 
             self._data_frame.set_index(idx_columns, inplace=True)
@@ -952,13 +1089,15 @@ class GroClient(object):
 
         return self._data_frame
 
-    def async_get_df(self):
-        self.batch_async_get_data_points(
-            self._data_series_queue,
-            output_list=self._data_frame,
-            map_result=self.add_points_to_df,
+    def async_get_df(self, show_revisions=False, show_available_date=False, index_by_series=False, include_names=False, compress_format=False):
+        return self.get_df(
+            show_revisions,
+            show_available_date,
+            index_by_series,
+            include_names,
+            compress_format,
+            async_mode=True
         )
-        return self._data_frame
 
     def add_points_to_df(self, index, data_series, data_points, *args):
         """Add the given datapoints to a pandas dataframe.
@@ -1011,7 +1150,7 @@ class GroClient(object):
             [{  'start_date': '2017-01-01T00:00:00.000Z',
                 'end_date': '2017-12-31T00:00:00.000Z',
                 'value': 408913833.8019222, 'unit_id': 15,
-                'reporting_date': None, 'available_date': None,
+                'reporting_date': None,
                 'metric_id': 860032, 'item_id': 274, 'region_id': 1215,
                 'partner_region_id': 0, 'frequency_id': 9, 'source_id': 2,
                 'belongs_to': {
@@ -1047,7 +1186,7 @@ class GroClient(object):
             [{  'start_date': '2017-01-01T00:00:00.000Z',
                 'end_date': '2017-12-31T00:00:00.000Z',
                 'value': 408913833.8019222, 'unit_id': 15,
-                'reporting_date': None, 'available_date': None,
+                'reporting_date': None,
                 'metric_id': 860032, 'item_id': 274, 'region_id': 1215,
                 'partner_region_id': 0, 'frequency_id': 9, 'source_id': 2,
                 'belongs_to': {
@@ -1060,7 +1199,7 @@ class GroClient(object):
             }, { 'start_date': '2017-01-01T00:00:00.000Z',
                  'end_date': '2017-12-31T00:00:00.000Z',
                  'value': 340614.19507563586, 'unit_id': 15,
-                 'reporting_date': None, 'available_date': None,
+                 'reporting_date': None,
                  'metric_id': 860032, 'item_id': 274, 'region_id': 1216,
                  'partner_region_id': 0, 'frequency_id': 9, 'source_id': 2,
                  'belongs_to': {
@@ -1105,6 +1244,8 @@ class GroClient(object):
             :sample:`at-time-query-examples.ipynb` for more details.
         include_historical : boolean, optional
             True by default, will include historical regions that are part of your selections
+        available_since : string, optional
+            Fetch points since last data retrieval where available date is equal to or after this date
 
         Returns
         -------
@@ -1302,6 +1443,8 @@ class GroClient(object):
                 data_series.pop("source_name", None)
                 # metadata is not hashable
                 data_series.pop("metadata", None)
+                # estimated data count is not hashable
+                data_series.pop("data_count_estimate", None)
                 series_hash = frozenset(data_series.items())
                 if series_hash not in ranking_groups:
                     ranking_groups.add(series_hash)
