@@ -4,8 +4,9 @@ try:
 except ImportError:
     # Python 2.7
     from mock import patch, MagicMock
-from unittest import TestCase
 from datetime import date
+import os
+from unittest import TestCase
 
 from groclient import GroClient
 from groclient.utils import zip_selections
@@ -109,6 +110,26 @@ def mock_get_descendant(
         return [{"id": child["id"]} for child in childs]
 
 
+def mock_get_ancestor(
+    access_token,
+    api_host,
+    entity_type,
+    entity_id,
+    distance,
+    include_details,
+):
+
+    childs = [
+            child
+            for child in mock_entities[entity_type].values()
+            if 12345 in child["contains"]
+        ]
+    if include_details:
+        return childs
+    else:
+        return [{"id": child["id"]} for child in childs]
+
+
 def mock_get_descendant_regions(
     access_token,
     api_host,
@@ -215,6 +236,7 @@ def mock_get_data_points(access_token, api_host, **selections):
 @patch("groclient.lib.get_geo_centre", MagicMock(side_effect=mock_get_geo_centre))
 @patch("groclient.lib.get_geojsons", MagicMock(side_effect=mock_get_geojsons))
 @patch("groclient.lib.get_geojson", MagicMock(side_effect=mock_get_geojson))
+@patch("groclient.lib.get_ancestor", MagicMock(side_effect=mock_get_ancestor))
 @patch("groclient.lib.get_descendant", MagicMock(side_effect=mock_get_descendant))
 @patch(
     "groclient.lib.get_descendant_regions",
@@ -302,6 +324,13 @@ class GroClientTests(TestCase):
         self.assertTrue("coordinates" in geojson["geometries"][0])
         self.assertTrue(geojson["geometries"][0]['coordinates'][0][0][0] == [-38, -4])
 
+    def test_get_ancestor(self):
+        self.assertTrue("name" in self.client.get_descendant('metrics', 119)[0])
+        self.assertTrue(
+            "name"
+            not in self.client.get_ancestor('regions', 12345, include_details=False)[0]
+        )
+
     def test_get_descendant(self):
         self.assertTrue("name" in self.client.get_descendant('metrics', 119)[0])
         self.assertTrue(
@@ -507,3 +536,36 @@ class GroClientTests(TestCase):
 
         with self.assertRaises(Exception):
             self.client.convert_unit({"value": None, "unit_id": 10}, 43)
+
+class GroClientConstructorTests(TestCase):
+    PROD_API_HOST = "api.gro-intelligence.com"
+
+    # The most convenient method.
+    @patch.dict(os.environ, {'GROAPI_TOKEN': MOCK_TOKEN})
+    def test_no_host_and_env_token(self):
+        client = GroClient()
+        with patch("groclient.lib.get_available") as get_available:
+            _ = client.get_available("items")
+            get_available.assert_called_once_with(MOCK_TOKEN, self.PROD_API_HOST, "items")
+
+    # A common use case: user passes an API token but no API host.
+    def test_no_host_and_kwarg_token(self):
+        client = GroClient(access_token=MOCK_TOKEN)
+        with patch("groclient.lib.get_available") as get_available:
+            _ = client.get_available("items")
+            get_available.assert_called_once_with(MOCK_TOKEN, self.PROD_API_HOST, "items")
+
+    def test_explicit_host_and_token(_self):
+        client = GroClient(MOCK_HOST, MOCK_TOKEN)
+        with patch("groclient.lib.get_available") as get_available:
+            _ = client.get_available("items")
+            get_available.assert_called_once_with(MOCK_TOKEN, MOCK_HOST, "items")
+
+    def test_missing_token(self):
+        # Explicitly unset GROAPI_TOKEN if it's set (eg, its set in our Shippable config).
+        env_without_token = {k: v for k, v in os.environ.items() if k != "GROAPI_TOKEN"}
+        with patch.dict(os.environ, env_without_token, clear=True):
+            with self.assertRaisesRegex(RuntimeError, "environment variable must be set"):
+                _ = GroClient(MOCK_HOST)
+            with self.assertRaisesRegex(RuntimeError, "environment variable must be set"):
+                _ = GroClient()
